% Create integral histogram from color image
function int_hist = compute_integral_color_histogram(img, nbins)
% histogram is 3D

col1 = img(:,:,1);
col2 = img(:,:,2);
col3 = img(:,:,3);

binwidth1 = 256/nbins(1);
binwidth2 = 256/nbins(2);
binwidth3 = 256/nbins(3);

int_hist = zeros(prod(size(img)), prod(nbins));

im1 = col1(:);
im2 = col2(:);
im3 = col3(:);
indx1 = floor( im1 / binwidth1 ) + 1;
indx2 = floor( im2 / binwidth2 ) + 1;
indx3 = floor( im3 / binwidth3 ) + 1;
if any(indx1<1 | indx1>nbins(1)), error('assert'), end
if any(indx2<1 | indx2>nbins(2)), error('assert'), end
if any(indx3<1 | indx3>nbins(3)), error('assert'), end

i = 1;
height = size(img, 1);
for c=1:size(img,2)
    for r=1:size(img,1)
        if r>1 & c>1,
            q_tl = GetIndex(r-1, c-1, height);
            q_t  = GetIndex(r-1, c, height);
            q_l  = GetIndex(r, c-1, height);
            int_hist(i,:) = int_hist(q_t,:) + int_hist(q_l,:) - int_hist(q_tl,:);
        end
        j = GetHistIndx(indx1(i), indx2(i), indx3(i), nbins);
        int_hist(i,j) = int_hist(i,j) + 1;
        i = i + 1;
    end
end

function indx = GetIndex(r, c, height)
indx = (c-1)*height + r;

function indx = GetHistIndx(indx1, indx2, indx3, nbins)
indx = (indx1-1) * (nbins(2)*nbins(3)) + (indx2-1) * (nbins(3)) + indx3;
