% input:  size of image (2-element vector)
%         state (r,c,s) of ellipse (center row and column, and length of horiz
%               axis
%         aspect_ratio:  ratio of vert axis to horiz axis
% output: perim:  nx2 list of points on perimeter; each row is (r,c) coords
%                 of pt; perimeter is just inside the mask
%         mask:  binary mask with ones in interior of ellipse
function [perim, mask] = generate_ellipse(img_sz, state, aspect_ratio)

if length(img_sz)==3,
    if img_sz(3)~=3, error('color image should have three channels')
    else img_sz = [img_sz(1:2)];  % retain only first two elements
    end
end
if length(img_sz)~=2, error('should be a 2x1 or 1x2 size'), end

axis_horiz = state(3);
axis_vert = round(state(3) * aspect_ratio);

% first compute all points as r=f(c)
c = -axis_horiz:axis_horiz;
rr = axis_vert * sqrt(1-(c / axis_horiz).^2);
ptsr = [rr -rr ; c c];

% then compute points as c=f(r)
r = -axis_vert:axis_vert;
cc = axis_horiz * sqrt(1-(r / axis_vert).^2);
ptsc = [r r ; cc -cc];

% combine into 2xN matrix
pts = [ptsr ptsc];

%figure(3), clf, plot(pts(1,:),pts(2,:),'o')

% splat all points onto a blank image, then fill it in to get mask
blank = zeros(img_sz);
for i=1:size(pts,2),
    row = round(pts(1,i) + state(1));
    col = round(pts(2,i) + state(2));
%    [row col]
    blank(row,col) = 1;
end
mask = imfill(blank, 4, 'holes');

% now get perimeter
foo = imerode(mask, ones(3,3));
diff = mask ~= foo;
%diff_left = diff & [zeros(img_sz(1), state(2)-1) ones(img_sz(1), img_sz(2)-state(2)+1)];
[rr,cc] = find(diff);
perim = [rr cc];
