% Given an image with ellipse state, generates a color histogram of the pixels
% inside the ellipse, then randomly samples the cdf of the histogram to
% generate a new image with approximately the same histogram.  Used for
% some figures in the paper.
function newim = generate_image_from_histogram_of_image(im, state, aspect_ratio, nbins)

    % create perimeter and mask for ellipse
    [ellipse_perimeter, ellipse_mask] = generate_ellipse(size(im), state, aspect_ratio)
%    ellipse_perimeter = ellipse(state, aspect_ratio);
%    ellipse_mask = create_ellipse_binary_mask(im, ellipse_perimeter);
    
    % compute color histogram; vectorize and normalize to make PDF
    hist = compute_color_histogram(im, state, aspect_ratio, nbins);
    hpdf = hist(:);
    hpdf = hpdf / sum(hpdf);  % make it a pdf
    
    % generate cdf of histogram
    hcdf = zeros(size(hpdf));
    hcdf(1) = hpdf(1);  
    for i=2:length(hpdf),
        hcdf(i) = hcdf(i-1) + hpdf(i);
    end
    
    % generate new image using histogram as generative model
    % (i.e., by randomly sampling cdf of histogram)
    newim = zeros(size(im));
    for i=1:size(newim,1),
        for j=1:size(newim,2),
            % find bin(s) closest to a random sampling of cdf
            diff = abs(hcdf-rand(1));
            ii = find(diff == min(diff));
            indx = ii(1);
            % get color of 'indx' bin in the histogram
            indx3 = floor((indx-1) / (nbins(1)*nbins(2))) + 1;
            leftover = mod((indx-1), nbins(1)*nbins(2));
            indx2 = floor(leftover / nbins(1)) + 1;
            indx1 = mod(leftover, nbins(1)) + 1;
			binwidth1 = 256/nbins(1);
			binwidth2 = 256/nbins(2);
			binwidth3 = 256/nbins(3);
            color1 = binwidth1 * ( indx1 - 0.5);
            color2 = binwidth2 * ( indx2 - 0.5);
            color3 = binwidth3 * ( indx3 - 0.5);
            if (color1>255 | color2>255 | color3>255), error('assert'), end
            newim(i,j,1) = color1;
            newim(i,j,2) = color2;
            newim(i,j,3) = color3;
        end
    end
    for i=1:3,
        newim(:,:,i) = newim(:,:,i) .* ellipse_mask;        
    end

