% output:  spatiogram is a struct with 3 fields:
%            - hist  (3D histogram)
%            - sumx  (sum of x locations for each bin, same size as hist)
%            - sumy  (sum of y locations for each bin, same size as hist)
%            - sumx2  (sum of x^2 locations for each bin, same size as hist)
%            - sumy2  (sum of y^2 locations for each bin, same size as hist)
%           To compute mean, use meanx = sumx ./ hist and sumy ./ hist
%           To compute variance, use sumx2 ./ hist - meanx.^2
%                                and sumy2 ./ hist - meany.^2
function spatiogram = compute_color_spatiogram(img, state, aspect_ratio, nbins)

ellipse_perimeter = ellipse(state, aspect_ratio);
ellipse_mask = create_ellipse_binary_mask(img,ellipse_perimeter);
col1 = img(:,:,1);
col2 = img(:,:,2);
col3 = img(:,:,3);

binwidth1 = 256/nbins(1);
binwidth2 = 256/nbins(2);
binwidth3 = 256/nbins(3);
spatiogram.histogram = zeros(nbins(1),nbins(2),nbins(3));
spatiogram.sumx = zeros(nbins(1),nbins(2),nbins(3));
spatiogram.sumy = zeros(nbins(1),nbins(2),nbins(3));
spatiogram.sumx2 = zeros(nbins(1),nbins(2),nbins(3));
spatiogram.sumy2 = zeros(nbins(1),nbins(2),nbins(3));
% histogram2  = color histogram of current image
% histogram1  = color histogram of model
for r=1:size(img,1)
    for c=1:size(img,2)
        if (ellipse_mask(r, c) == 1)
            indx1 = floor( col1(r,c) / binwidth1 ) + 1;
            indx2 = floor( col2(r,c) / binwidth2 ) + 1;
            indx3 = floor( col3(r,c) / binwidth3 ) + 1;
            spatiogram.histogram(indx1, indx2, indx3) = spatiogram.histogram(indx1, indx2, indx3) + 1;
            spatiogram.sumx(indx1, indx2, indx3) = spatiogram.sumx(indx1, indx2, indx3) + c;
            spatiogram.sumy(indx1, indx2, indx3) = spatiogram.sumy(indx1, indx2, indx3) + r;
            spatiogram.sumx2(indx1, indx2, indx3) = spatiogram.sumx2(indx1, indx2, indx3) + c*c;
            spatiogram.sumy2(indx1, indx2, indx3) = spatiogram.sumy2(indx1, indx2, indx3) + r*r;
        end
    end
end